@@ -543,7 +543,6 @@ SourceBrowserPreferences::SourceBrowserPreferences(GtkWidget *pParent)
 
   if (!gpGuiProcessor && !gpGuiProcessor->source_browser)
     return;
-  printf("RRR SourceBrowserPreferences::SourceBrowserPreferences\n");
 
   GtkWidget *notebook = gtk_notebook_new();
   gtk_notebook_set_tab_pos((GtkNotebook*)notebook,GTK_POS_TOP);
@@ -1268,7 +1267,7 @@ cbTimeFormatPopup(GtkWidget *widget, GdkEventButton *event, TimeWidget *tw)
   if(!widget || !event || !tw)
     return 0;
 
-  if( (event->type == GDK_BUTTON_PRESS) ) {// &&  (event->button == 3) ) {
+  if(event->type == GDK_BUTTON_PRESS) {
 
     gtk_menu_popup(GTK_MENU(tw->menu), 0, 0, 0, 0,
                    3, event->time);
