@@ -1757,7 +1757,7 @@ gdouble Scope_Window::getSpan()
   return start > stop ? 0.0 : ((gdouble)(stop-start));
 }
 
-guint64 Scope_Window::mapPixelToTime(unsigned int pixel)
+guint64 Scope_Window::mapPixelToTime(int pixel)
 {
   gdouble x = (pixel>=0 && pixel<m_PixmapWidth) ? pixel : 0;
 
