@@ -162,6 +162,19 @@ void  BSR::put_value(unsigned int new_value)
   cpu_pic->indf->update();
 }
 
+void  IOCxF::put(unsigned int new_value)
+{
+  unsigned int masked_value = new_value & mValidBits;
+  Dprintf((" %s value %x add %x\n", name().c_str(), new_value, new_value&valid_bits));
+
+  get_trace().raw(write_trace.get() | value.get());
+  value.put(masked_value);
+  if (intcon)
+  {
+    ((INTCON_14_PIR *)intcon)->set_rbif(masked_value != 0);
+    ((INTCON_14_PIR *)intcon)->aocxf_val(this, masked_value);
+  }
+}
 
 // Adjust internal RC oscillator frequency as per 12f675/629
 // Spec sheet does not give range so assume +/- 12.5% as per 16f88
@@ -179,8 +192,16 @@ void  OSCCAL::put(unsigned int new_value)
   if (base_freq > 0.)
   {
     adj  = adj -  0x80;
+    // A hook to honour configured frequency - if we're going to change it now
+    if(cpu_pic->get_frequency() > base_freq*0.875 && base_freq*1.125 > cpu_pic->get_frequency()) {
+      base_freq=cpu_pic->get_frequency();
+      if (verbose)
+        cout << "Adjusting base frequency for INTOSC calibration: " << base_freq << "\n";
+    }
     tune = (1. + 0.125 * adj / 0x80) * base_freq;
     cpu_pic->set_frequency(tune);
+    if (verbose)
+      cout << "Calibrating INTOSC by " << adj << " to " << tune << "\n";
   }
 }
 
@@ -203,7 +224,7 @@ void OSCCON::callback()
 }
 bool OSCCON::set_rc_frequency()
 {
-  double base_frequency = 31.25e3;
+  double base_frequency = 31.e3;
 
 
   if (!cpu_pic->get_int_osc())
@@ -213,7 +234,7 @@ bool OSCCON::set_rc_frequency()
     switch (new_IRCF)
     {
     case 0:
-	base_frequency = 31.25e3;
+	base_frequency = 31.e3;
 	break;
 	
     case 1:
