@@ -161,8 +161,9 @@ void P18C2x2::create()
     cout << "P18C2x2::create\n";
 
   create_iopin_map();
-
   _16bit_compat_adc::create();
+  add_sfr_register(&osccon, 0xfd3, RegisterValue(0x00,0), "osccon");
+  init_pir2(pir2, PIR2v2::TMR3IF);
 
 }
 
@@ -349,6 +350,7 @@ void P18C4x2::create()
   create_iopin_map();
 
   _16bit_compat_adc::create();
+  add_sfr_register(&osccon, 0xfd3, RegisterValue(0x00,0), "osccon");
 
 }
 //------------------------------------------------------------------------
@@ -409,6 +411,7 @@ void P18C4x2::create_iopin_map()
   package->assign_pin(39, m_portb->addPin(new IO_bi_directional_pu("portb6"),6));
   package->assign_pin(40, m_portb->addPin(new IO_bi_directional_pu("portb7"),7));
 
+
   psp.initialize(&pir_set_def,    // PIR
                 m_portd,           // Parallel port
                 m_trisd,           // Parallel tris
@@ -494,6 +497,8 @@ void P18C4x2::create_sfr_map()
   adcon1->setIOPin(5, &(*m_porte)[0]);
   adcon1->setIOPin(6, &(*m_porte)[1]);
   adcon1->setIOPin(7, &(*m_porte)[2]);
+  init_pir2(pir2, PIR2v2::TMR3IF); 
+  tmr3l.setIOpin(&(*m_portc)[0]);
 
   //1 usart16.initialize_16(this,&pir_set_def,&portc);
 
@@ -606,7 +611,7 @@ void P18F242::create()
   if(verbose)
     cout << " 18f242 create \n";
 
-  e = new EEPROM_PIR(this,&pir2);
+  e = new EEPROM_PIR(this,pir2);
 
   // We might want to pass this value in for larger eeproms
   e->initialize(256);
@@ -711,7 +716,7 @@ void P18F442::create()
   if(verbose)
     cout << " 18f442 create \n";
 
-  e = new EEPROM_PIR(this,&pir2);
+  e = new EEPROM_PIR(this,pir2);
 
   // We might want to pass this value in for larger eeproms
   e->initialize(256);
@@ -899,8 +904,7 @@ P18F2455::P18F2455(const char *_name, const char *desc)
       uep12(this, "uep12", "USB Endpoint 12 Enable register"    ),
       uep13(this, "uep13", "USB Endpoint 13 Enable register"    ),
       uep14(this, "uep14", "USB Endpoint 14 Enable register"    ),
-      uep15(this, "uep15", "USB Endpoint 15 Enable register"    ),
-      pir2 (this, "pir2" , "Peripheral Interrupt Register",0,0  )
+      uep15(this, "uep15", "USB Endpoint 15 Enable register"    )
 {
 
   cout << "\nP18F2455 does not support USB registers and functionality\n\n";
@@ -937,7 +941,6 @@ P18F2455::~P18F2455()
   remove_sfr_register(&uep13);
   remove_sfr_register(&uep14);
   remove_sfr_register(&uep15);
-  remove_sfr_register(&pir2 );
 }
 void P18F2455::create_sfr_map()
 {
@@ -985,18 +988,6 @@ void P18F2455::create_sfr_map()
   add_sfr_register(&uep13,0x0F7D, RegisterValue(0,0));
   add_sfr_register(&uep14,0x0F7E, RegisterValue(0,0));
   add_sfr_register(&uep15,0x0F7F, RegisterValue(0,0));
-
-  // Initialize the register cross linkages
-  pir_set_def.set_pir2(&pir2);
-  delete tmr3l.getInterruptSource();
-  tmr3l.setInterruptSource(new InterruptSource(&pir2, PIR2v4::TMR3IF));
-  pir2.set_intcon(&intcon);
-  pir2.set_pie(&pie2);
-  pir2.set_ipr(&ipr2);
-  pie2.setPir(&pir2);
-  //pie2.new_name("pie2");
-
-  // new InterruptSource(&pir2, PIR2v4::USBIF);
 }
 
 Processor * P18F2455::construct(const char *name)
@@ -1013,6 +1004,138 @@ Processor * P18F2455::construct(const char *name)
   return p;
 }
 
+//------------------------------------------------------------------------
+//
+// P18F2550	- 28 pin
+// 
+
+P18F2550::P18F2550(const char *_name, const char *desc)
+  : P18F2x21(_name,desc),
+      ufrml(this, "ufrml", "USB Frame Number register Low"      ),
+      ufrmh(this, "ufrmh", "USB Frame Number register High"     ),
+      uir  (this, "uir"  , "USB Interrupt Status register"      ),
+      uie  (this, "uie"  , "USB Interrupt Enable register"      ),
+      ueir (this, "ueir" , "USB Error Interrupt Status register"),
+      ueie (this, "ueie" , "USB Error Interrupt Enable register"),
+      ustat(this, "ustat", "USB Transfer Status register"       ),
+      ucon (this, "ucon" , "USB Control register"               ),
+      uaddr(this, "uaddr", "USB Device Address register"        ),
+      ucfg (this, "ucfg" , "USB Configuration register"         ),
+      uep0 (this, "uep0" , "USB Endpoint 0 Enable register"     ),
+      uep1 (this, "uep1" , "USB Endpoint 1 Enable register"     ),
+      uep2 (this, "uep2" , "USB Endpoint 2 Enable register"     ),
+      uep3 (this, "uep3" , "USB Endpoint 3 Enable register"     ),
+      uep4 (this, "uep4" , "USB Endpoint 4 Enable register"     ),
+      uep5 (this, "uep5" , "USB Endpoint 5 Enable register"     ),
+      uep6 (this, "uep6" , "USB Endpoint 6 Enable register"     ),
+      uep7 (this, "uep7" , "USB Endpoint 7 Enable register"     ),
+      uep8 (this, "uep8" , "USB Endpoint 8 Enable register"     ),
+      uep9 (this, "uep9" , "USB Endpoint 9 Enable register"     ),
+      uep10(this, "uep10", "USB Endpoint 10 Enable register"    ),
+      uep11(this, "uep11", "USB Endpoint 11 Enable register"    ),
+      uep12(this, "uep12", "USB Endpoint 12 Enable register"    ),
+      uep13(this, "uep13", "USB Endpoint 13 Enable register"    ),
+      uep14(this, "uep14", "USB Endpoint 14 Enable register"    ),
+      uep15(this, "uep15", "USB Endpoint 15 Enable register"    )
+{
+
+  cout << "\nP18F2550 does not support USB registers and functionality\n\n";
+  if(verbose)
+    cout << "18f2550 constructor, type = " << isa() << '\n';
+}
+
+P18F2550::~P18F2550()
+{
+  remove_sfr_register(&ufrml);
+  remove_sfr_register(&ufrmh);
+  remove_sfr_register(&uir  );
+  remove_sfr_register(&uie  );
+  remove_sfr_register(&ueir );
+  remove_sfr_register(&ueie );
+  remove_sfr_register(&ustat);
+  remove_sfr_register(&ucon );
+  remove_sfr_register(&uaddr);
+  remove_sfr_register(&ucfg );
+  remove_sfr_register(&uep0 );
+  remove_sfr_register(&uep1 );
+  remove_sfr_register(&uep2 );
+  remove_sfr_register(&uep3 );
+  remove_sfr_register(&uep4 );
+  remove_sfr_register(&uep5 );
+  remove_sfr_register(&uep6 );
+  remove_sfr_register(&uep7 );
+  remove_sfr_register(&uep8 );
+  remove_sfr_register(&uep9 );
+  remove_sfr_register(&uep10);
+  remove_sfr_register(&uep11);
+  remove_sfr_register(&uep12);
+  remove_sfr_register(&uep13);
+  remove_sfr_register(&uep14);
+  remove_sfr_register(&uep15);
+}
+void P18F2550::create_sfr_map()
+{
+
+  if(verbose)
+    cout << " 18f2550 create_sfr_map \n";
+
+
+  P18F2x21::create_sfr_map();
+  package->destroy_pin(14);
+  package->assign_pin(14, 0, false);          // Vusb
+
+  /* The MSSP/I2CC pins are different on this chip. */
+  ssp.initialize(&pir_set_def,         // PIR
+                 &(*m_portb)[1],       // SCK
+                 &(*m_porta)[5],       // SS
+                 &(*m_portc)[7],       // SDO
+                 &(*m_portb)[0],       // SDI
+                 m_trisb,              // i2c tris port
+                 SSP_TYPE_MSSP
+       );
+  add_sfr_register(&ufrml,0x0F66, RegisterValue(0,0),"ufrm");
+  add_sfr_register(&ufrmh,0X0F67, RegisterValue(0,0));
+  add_sfr_register(&uir  ,0x0F68, RegisterValue(0,0));
+  add_sfr_register(&uie  ,0x0F69, RegisterValue(0,0));
+  add_sfr_register(&ueir ,0x0F6A, RegisterValue(0,0));
+  add_sfr_register(&ueie ,0x0F6B, RegisterValue(0,0));
+  add_sfr_register(&ustat,0X0F6C, RegisterValue(0,0));
+  add_sfr_register(&ucon ,0x0F6D, RegisterValue(0,0));
+  add_sfr_register(&uaddr,0X0F6E, RegisterValue(0,0));
+  add_sfr_register(&ucfg ,0x0F6F, RegisterValue(0,0));
+  add_sfr_register(&uep0 ,0x0F70, RegisterValue(0,0));
+  add_sfr_register(&uep1 ,0x0F71, RegisterValue(0,0));
+  add_sfr_register(&uep2 ,0x0F72, RegisterValue(0,0));
+  add_sfr_register(&uep3 ,0x0F73, RegisterValue(0,0));
+  add_sfr_register(&uep4 ,0x0F74, RegisterValue(0,0));
+  add_sfr_register(&uep5 ,0x0F75, RegisterValue(0,0));
+  add_sfr_register(&uep6 ,0x0F76, RegisterValue(0,0));
+  add_sfr_register(&uep7 ,0x0F77, RegisterValue(0,0));
+  add_sfr_register(&uep8 ,0x0F78, RegisterValue(0,0));
+  add_sfr_register(&uep9 ,0x0F79, RegisterValue(0,0));
+  add_sfr_register(&uep10,0x0F7A, RegisterValue(0,0));
+  add_sfr_register(&uep11,0x0F7B, RegisterValue(0,0));
+  add_sfr_register(&uep12,0x0F7C, RegisterValue(0,0));
+  add_sfr_register(&uep13,0x0F7D, RegisterValue(0,0));
+  add_sfr_register(&uep14,0x0F7E, RegisterValue(0,0));
+  add_sfr_register(&uep15,0x0F7F, RegisterValue(0,0));
+
+}
+
+Processor * P18F2550::construct(const char *name)
+{
+
+  P18F2550 *p = new P18F2550(name);
+
+  if(verbose)
+    cout << " 18F2550 construct\n";
+
+  p->create();
+  p->create_invalid_registers();
+  p->create_symbols();
+  return p;
+}
+
 //------------------------------------------------------------------------
 //
 // P18F4455
@@ -1046,7 +1169,10 @@ P18F4455::P18F4455(const char *_name, const char *desc)
       uep13(this, "uep13", "USB Endpoint 13 Enable register"    ),
       uep14(this, "uep14", "USB Endpoint 14 Enable register"    ),
       uep15(this, "uep15", "USB Endpoint 15 Enable register"    ),
-      pir2 (this, "pir2" , "Peripheral Interrupt Register",0,0  )
+      sppcon(this, "sppcon", "Streaming Parallel port control register"),
+      sppcfg(this, "sppcfg", "Streaming Parallel port configuration register"),
+      sppeps(this, "sppeps", "SPP ENDPOINT ADDRESS AND STATUS REGISTER"),
+      sppdata(this, "sppdata", "Streaming Parallel port data register")
 
 {
   cout << "\nP18F4455 does not support USB registers and functionality\n\n";
@@ -1082,6 +1208,10 @@ P18F4455::~P18F4455()
   remove_sfr_register(&uep13);
   remove_sfr_register(&uep14);
   remove_sfr_register(&uep15);
+  remove_sfr_register(&sppcon);
+  remove_sfr_register(&sppcfg);
+  remove_sfr_register(&sppeps);
+  remove_sfr_register(&sppdata);
 }
 
 void P18F4455::create()
@@ -1105,6 +1235,24 @@ void P18F4455::create()
        );
 
   // RP: RRR commented out comparator.cmcon.set_eccpas(&eccpas);  ??
+  // Streaming Parallel port (SPP)
+  spp.initialize(&pir_set_def,         // PIR
+		m_portd,		//Parallel port
+		m_trisd,		//Parallel port tristate register
+		&sppcon,
+		&sppcfg,
+		&sppeps,
+		&sppdata,
+		&(*m_porte)[0],		// CLK1SPP
+		&(*m_porte)[1],		// CLK2SPP
+		&(*m_porte)[2],		// OESPP
+		&(*m_portb)[4]		// CSSPP
+	);
+
+  add_sfr_register(&sppdata,0x0F62, RegisterValue(0,0));
+  add_sfr_register(&sppcfg,0x0F63, RegisterValue(0,0));
+  add_sfr_register(&sppeps,0x0F64, RegisterValue(0,0));
+  add_sfr_register(&sppcon,0x0F65, RegisterValue(0,0));
   add_sfr_register(&ufrml,0x0F66, RegisterValue(0,0),"ufrm");
   add_sfr_register(&ufrmh,0X0F67, RegisterValue(0,0));
   add_sfr_register(&uir  ,0x0F68, RegisterValue(0,0));
@@ -1133,16 +1281,9 @@ void P18F4455::create()
   add_sfr_register(&uep15,0x0F7F, RegisterValue(0,0));
 
   // Initialize the register cross linkages
-  pir_set_def.set_pir2(&pir2);
-  delete tmr3l.getInterruptSource();
-  tmr3l.setInterruptSource(new InterruptSource(&pir2, PIR2v4::TMR3IF));
-  pir2.set_intcon(&intcon);
-  pir2.set_pie(&pie2);
-  pir2.set_ipr(&ipr2);
-  pie2.setPir(&pir2);
-  //pie2.new_name("pie2");
+  init_pir2(pir2, PIR2v4::TMR3IF);
 
-  //new InterruptSource(&pir2, PIR2v4::USBIF);
+  //new InterruptSource(pir2, PIR2v4::USBIF);
 }
 
 Processor * P18F4455::construct(const char *name)
@@ -1160,16 +1301,173 @@ Processor * P18F4455::construct(const char *name)
 }
 
 
-
-
-
 //------------------------------------------------------------------------
+//
+// P18F4550
+// 
+
+P18F4550::P18F4550(const char *_name, const char *desc)
+  : P18F4x21(_name,desc),
+      ufrml(this, "ufrml", "USB Frame Number register Low"      ),
+      ufrmh(this, "ufrmh", "USB Frame Number register High"     ),
+      uir  (this, "uir"  , "USB Interrupt Status register"      ),
+      uie  (this, "uie"  , "USB Interrupt Enable register"      ),
+      ueir (this, "ueir" , "USB Error Interrupt Status register"),
+      ueie (this, "ueie" , "USB Error Interrupt Enable register"),
+      ustat(this, "ustat", "USB Transfer Status register"       ),
+      ucon (this, "ucon" , "USB Control register"               ),
+      uaddr(this, "uaddr", "USB Device Address register"        ),
+      ucfg (this, "ucfg" , "USB Configuration register"         ),
+      uep0 (this, "uep0" , "USB Endpoint 0 Enable register"     ),
+      uep1 (this, "uep1" , "USB Endpoint 1 Enable register"     ),
+      uep2 (this, "uep2" , "USB Endpoint 2 Enable register"     ),
+      uep3 (this, "uep3" , "USB Endpoint 3 Enable register"     ),
+      uep4 (this, "uep4" , "USB Endpoint 4 Enable register"     ),
+      uep5 (this, "uep5" , "USB Endpoint 5 Enable register"     ),
+      uep6 (this, "uep6" , "USB Endpoint 6 Enable register"     ),
+      uep7 (this, "uep7" , "USB Endpoint 7 Enable register"     ),
+      uep8 (this, "uep8" , "USB Endpoint 8 Enable register"     ),
+      uep9 (this, "uep9" , "USB Endpoint 9 Enable register"     ),
+      uep10(this, "uep10", "USB Endpoint 10 Enable register"    ),
+      uep11(this, "uep11", "USB Endpoint 11 Enable register"    ),
+      uep12(this, "uep12", "USB Endpoint 12 Enable register"    ),
+      uep13(this, "uep13", "USB Endpoint 13 Enable register"    ),
+      uep14(this, "uep14", "USB Endpoint 14 Enable register"    ),
+      uep15(this, "uep15", "USB Endpoint 15 Enable register"    ),
+      sppcon(this, "sppcon", "Streaming Parallel port control register"),
+      sppcfg(this, "sppcfg", "Streaming Parallel port configuration register"),
+      sppeps(this, "sppeps", "SPP ENDPOINT ADDRESS AND STATUS REGISTER"),
+      sppdata(this, "sppdata", "Streaming Parallel port data register")
 
-P18Fxx20::P18Fxx20(const char *_name, const char *desc)
-  : _16bit_v2_adc(_name,desc)
 {
+  cout << "\nP18F4550 does not support USB registers and functionality\n\n";
+  if(verbose)
+    cout << "18f4550 constructor, type = " << isa() << '\n';
 }
 
+P18F4550::~P18F4550()
+{
+  remove_sfr_register(&ufrml);
+  remove_sfr_register(&ufrmh);
+  remove_sfr_register(&uir  );
+  remove_sfr_register(&uie  );
+  remove_sfr_register(&ueir );
+  remove_sfr_register(&ueie );
+  remove_sfr_register(&ustat);
+  remove_sfr_register(&ucon );
+  remove_sfr_register(&uaddr);
+  remove_sfr_register(&ucfg );
+  remove_sfr_register(&uep0 );
+  remove_sfr_register(&uep1 );
+  remove_sfr_register(&uep2 );
+  remove_sfr_register(&uep3 );
+  remove_sfr_register(&uep4 );
+  remove_sfr_register(&uep5 );
+  remove_sfr_register(&uep6 );
+  remove_sfr_register(&uep7 );
+  remove_sfr_register(&uep8 );
+  remove_sfr_register(&uep9 );
+  remove_sfr_register(&uep10);
+  remove_sfr_register(&uep11);
+  remove_sfr_register(&uep12);
+  remove_sfr_register(&uep13);
+  remove_sfr_register(&uep14);
+  remove_sfr_register(&uep15);
+  remove_sfr_register(&sppcon);
+  remove_sfr_register(&sppcfg);
+  remove_sfr_register(&sppeps);
+  remove_sfr_register(&sppdata);
+}
+
+void P18F4550::create()
+{
+  P18F4x21::create();
+
+  if(verbose)
+    cout << " 18f4550 create \n";
+
+  package->destroy_pin(18);
+  package->assign_pin(18, 0, false);          // Vusb
+
+  /* The MSSP/I2CC pins are different on this chip. */
+  ssp.initialize(&pir_set_def,         // PIR
+                 &(*m_portb)[1],       // SCK
+                 &(*m_porta)[5],       // SS
+                 &(*m_portc)[7],       // SDO
+                 &(*m_portb)[0],       // SDI
+                 m_trisb,              // i2c tris port
+                 SSP_TYPE_MSSP
+       );
+
+  // Streaming Parallel port (SPP)
+  spp.initialize(&pir_set_def,         // PIR
+		m_portd,		//Parallel port
+		m_trisd,		//Parallel port tristate register
+		&sppcon,
+		&sppcfg,
+		&sppeps,
+		&sppdata,
+		&(*m_porte)[0],		// CLK1SPP
+		&(*m_porte)[1],		// CLK2SPP
+		&(*m_porte)[2],		// OESPP
+		&(*m_portb)[4]		// CSSPP
+	);
+
+  // RP: RRR commented out comparator.cmcon.set_eccpas(&eccpas);  ??
+  add_sfr_register(&sppdata,0x0F62, RegisterValue(0,0));
+  add_sfr_register(&sppcfg,0x0F63, RegisterValue(0,0));
+  add_sfr_register(&sppeps,0x0F64, RegisterValue(0,0));
+  add_sfr_register(&sppcon,0x0F65, RegisterValue(0,0));
+  add_sfr_register(&ufrml,0x0F66, RegisterValue(0,0),"ufrm");
+  add_sfr_register(&ufrmh,0X0F67, RegisterValue(0,0));
+  add_sfr_register(&uir  ,0x0F68, RegisterValue(0,0));
+  add_sfr_register(&uie  ,0x0F69, RegisterValue(0,0));
+  add_sfr_register(&ueir ,0x0F6A, RegisterValue(0,0));
+  add_sfr_register(&ueie ,0x0F6B, RegisterValue(0,0));
+  add_sfr_register(&ustat,0X0F6C, RegisterValue(0,0));
+  add_sfr_register(&ucon ,0x0F6D, RegisterValue(0,0));
+  add_sfr_register(&uaddr,0X0F6E, RegisterValue(0,0));
+  add_sfr_register(&ucfg ,0x0F6F, RegisterValue(0,0));
+  add_sfr_register(&uep0 ,0x0F70, RegisterValue(0,0));
+  add_sfr_register(&uep1 ,0x0F71, RegisterValue(0,0));
+  add_sfr_register(&uep2 ,0x0F72, RegisterValue(0,0));
+  add_sfr_register(&uep3 ,0x0F73, RegisterValue(0,0));
+  add_sfr_register(&uep4 ,0x0F74, RegisterValue(0,0));
+  add_sfr_register(&uep5 ,0x0F75, RegisterValue(0,0));
+  add_sfr_register(&uep6 ,0x0F76, RegisterValue(0,0));
+  add_sfr_register(&uep7 ,0x0F77, RegisterValue(0,0));
+  add_sfr_register(&uep8 ,0x0F78, RegisterValue(0,0));
+  add_sfr_register(&uep9 ,0x0F79, RegisterValue(0,0));
+  add_sfr_register(&uep10,0x0F7A, RegisterValue(0,0));
+  add_sfr_register(&uep11,0x0F7B, RegisterValue(0,0));
+  add_sfr_register(&uep12,0x0F7C, RegisterValue(0,0));
+  add_sfr_register(&uep13,0x0F7D, RegisterValue(0,0));
+  add_sfr_register(&uep14,0x0F7E, RegisterValue(0,0));
+  add_sfr_register(&uep15,0x0F7F, RegisterValue(0,0));
+
+  // Initialize the register cross linkages
+
+  //new InterruptSource(pir2, PIR2v4::USBIF);
+}
+
+Processor * P18F4550::construct(const char *name)
+{
+
+  P18F4550 *p = new P18F4550(name);
+
+  if(verbose)
+    cout << " 18F4550 construct\n";
+
+  p->create();
+  p->create_invalid_registers();
+  p->create_symbols();
+  return p;
+}
+
+
+
+
+
 //------------------------------------------------------------------------
 //
 // P18F1220
@@ -1196,7 +1494,7 @@ void P18F1220::create()
   if(verbose)
     cout << "P18F1220::create\n";
 
-  e = new EEPROM_PIR(this,&pir2);
+  e = new EEPROM_PIR(this,pir2);
 
   // We might want to pass this value in for larger eeproms
   e->initialize(256);
@@ -1211,6 +1509,7 @@ void P18F1220::create()
 
   _16bit_processor::create();
   _16bit_v2_adc::create(7);
+  add_sfr_register(&osccon, 0xfd3, RegisterValue(0x00,0), "osccon");
   usart.txsta.setIOpin(&(*m_portb)[1]);
   usart.rcsta.setIOpin(&(*m_portb)[4]);
   adcon1->setIOPin(4, &(*m_portb)[0]);
@@ -1247,6 +1546,8 @@ void P18F1220::create()
   eccpas.link_registers(&pwm1con, &ccp1con);
   ccp1con.pwm1con = &pwm1con;
   ccp1con.setIOpin(&((*m_portb)[3]), &((*m_portb)[2]), &((*m_portb)[6]), &((*m_portb)[7]));
+  init_pir2(pir2, PIR2v2::TMR3IF);
+  tmr3l.setIOpin(&(*m_portb)[6]);
 }
 //------------------------------------------------------------------------
 void P18F1220::create_iopin_map()
@@ -1281,7 +1582,7 @@ void P18F1220::create_iopin_map()
 }
 
 P18F1220::P18F1220(const char *_name, const char *desc)
-  : P18Fxx20(_name,desc),
+  : _16bit_v2_adc(_name,desc),
     osctune(this, "osctune", "OSC Tune"),
     eccpas(this, "eccpas", "ECCP Auto-Shutdown Control Register"),
     pwm1con(this, "pwm1con", "Enhanced PWM Control Register")
@@ -1419,7 +1720,9 @@ void P18F2x21::create()
   if(verbose)
     cout << "P18F2x21::create\n";
 
-  e = new EEPROM_PIR(this,&pir2);
+  delete pir2;
+  pir2 = (PIR2v2 *)(new PIR2v4(this, "pir2" , "Peripheral Interrupt Register",0,0  ));
+  e = new EEPROM_PIR(this,pir2);
   e->initialize ( eeprom_memory_size() );
   e->set_intcon(&intcon);
   // assign this eeprom to the processor
@@ -1428,6 +1731,7 @@ void P18F2x21::create()
   create_iopin_map();
 
   _16bit_processor::create();
+  add_sfr_register(&osccon, 0xfd3, RegisterValue(0x40,0), "osccon");
 
   m_configMemory->addConfigWord(CONFIG3H-CONFIG1L,new Config3H_2x21(this, CONFIG3H, 0x83));
   m_configMemory->addConfigWord(CONFIG1H-CONFIG1L,new Config1H_4bits(this, CONFIG1H, 0x07));
@@ -1435,10 +1739,13 @@ void P18F2x21::create()
   set_osc_pin_Number(0, 9, &(*m_porta)[7]);
   set_osc_pin_Number(1,10, &(*m_porta)[6]);
 
+
+
   /// @bug registers not present on 28 pin according to table 5-1 of the
   /// data sheet, but bit-restricted according to section 16.4.7
   add_sfr_register(&pwm1con, 0xfb7, RegisterValue(0,0));
-  add_sfr_register(&eccpas, 0xfb6, RegisterValue(0,0));
+  add_sfr_register(&eccpas, 0xfb6, RegisterValue(0,0));  
+
   eccpas.setBitMask(0xfc);
   eccpas.setIOpin(0, 0, &(*m_portb)[0]);
   eccpas.link_registers(&pwm1con, &ccp1con);
@@ -1604,7 +1911,7 @@ void P18F2x21::create_sfr_map()
   add_sfr_register(&comparator.cmcon, 0xfb4, RegisterValue(7,0),"cmcon");
   add_sfr_register(&comparator.vrcon, 0xfb5, RegisterValue(0,0),"cvrcon");
 
-  ccp2con.setCrosslinks(&ccpr2l, &pir2, PIR2v2::CCP2IF, &tmr2);
+  ccp2con.setCrosslinks(&ccpr2l, pir2, PIR2v2::CCP2IF, &tmr2);
 //  ccp2con.setIOpin(&((*m_portc)[1]));     // handled by Config3H_2x21
   ccpr2l.ccprh  = &ccpr2h;
   ccpr2l.tmrl   = &tmr1l;
@@ -1614,6 +1921,8 @@ void P18F2x21::create_sfr_map()
   add_sfr_register(&usart.spbrgh,   0xfb0,porv,"spbrgh");
   add_sfr_register(&usart.baudcon,  0xfb8,porv,"baudcon");
   usart.set_eusart(true);
+  init_pir2(pir2, PIR2v4::TMR3IF); 
+  tmr3l.setIOpin(&(*m_portc)[0]);
 }
 
 void P18F2x21::osc_mode(unsigned int value)
@@ -1826,7 +2135,9 @@ void P18F4x21::create()
   if(verbose)
     cout << "P18F4x21::create\n";
 
-  e = new EEPROM_PIR(this,&pir2);
+  delete pir2;
+  pir2 = (PIR2v2 *)(new PIR2v4(this, "pir2" , "Peripheral Interrupt Register",0,0  ));
+  e = new EEPROM_PIR(this,pir2);
   e->initialize ( eeprom_memory_size() );
   e->set_intcon(&intcon);
   // assign this eeprom to the processor
@@ -1844,6 +2155,7 @@ void P18F4x21::create()
 
   add_sfr_register(&pwm1con, 0xfb7, RegisterValue(0,0));
   add_sfr_register(&eccpas, 0xfb6, RegisterValue(0,0));
+  add_sfr_register(&osccon, 0xfd3, RegisterValue(0x40,0), "osccon");
   eccpas.setIOpin(0, 0, &(*m_portb)[0]);
   eccpas.link_registers(&pwm1con, &ccp1con);
   comparator.cmcon.set_eccpas(&eccpas);
@@ -1913,6 +2225,7 @@ void P18F4x21::create_iopin_map()
   package->assign_pin(39, m_portb->addPin(new IO_bi_directional_pu("portb6"),6));
   package->assign_pin(40, m_portb->addPin(new IO_bi_directional_pu("portb7"),7));
 
+/*RRR
   psp.initialize(&pir_set_def,    // PIR
                 m_portd,           // Parallel port
                 m_trisd,           // Parallel tris
@@ -1920,7 +2233,7 @@ void P18F4x21::create_iopin_map()
                 &(*m_porte)[0],    // NOT RD
                 &(*m_porte)[1],    // NOT WR
                 &(*m_porte)[2]);   // NOT CS
-
+RRR*/
   tmr1l.setIOpin(&(*m_portc)[0]);
 
   ssp.initialize(&pir_set_def,    // PIR
@@ -2045,7 +2358,7 @@ void P18F4x21::create_sfr_map()
   add_sfr_register(&comparator.cmcon, 0xfb4, RegisterValue(7,0),"cmcon");
   add_sfr_register(&comparator.vrcon, 0xfb5, RegisterValue(0,0),"cvrcon");
 
-  ccp2con.setCrosslinks(&ccpr2l, &pir2, PIR2v2::CCP2IF, &tmr2);
+  ccp2con.setCrosslinks(&ccpr2l, pir2, PIR2v2::CCP2IF, &tmr2);
 //  ccp2con.setIOpin(&((*m_portc)[1]));     // Handled by Config3H_2x21::set
   ccpr2l.ccprh  = &ccpr2h;
   ccpr2l.tmrl   = &tmr1l;
@@ -2055,6 +2368,8 @@ void P18F4x21::create_sfr_map()
   add_sfr_register(&usart.spbrgh,   0xfb0,porv,"spbrgh");
   add_sfr_register(&usart.baudcon,  0xfb8,porv,"baudcon");
   usart.set_eusart(true);
+  init_pir2(pir2, PIR2v4::TMR3IF); 
+  tmr3l.setIOpin(&(*m_portc)[0]);
 }
 
 
@@ -2235,7 +2550,7 @@ void P18F6x20::create()
   if(verbose)
     cout << "P18F6x20::create\n";
 
-  e = new EEPROM_PIR(this,&pir2);
+  e = new EEPROM_PIR(this,pir2);
   e->initialize ( eeprom_memory_size() );
   e->set_intcon(&intcon);
   // assign this eeprom to the processor
@@ -2244,8 +2559,11 @@ void P18F6x20::create()
   create_iopin_map();
 
   _16bit_processor::create();
+  add_sfr_register(&osccon, 0xfd3, RegisterValue(0x40,0), "osccon");
 
   m_configMemory->addConfigWord(CONFIG1H-CONFIG1L,new Config1H_4bits(this, CONFIG1H, 0x27));
+  init_pir2(pir2, PIR2v2::TMR3IF);
+  tmr3l.setIOpin(&(*m_portc)[0]);
 }
 
 //------------------------------------------------------------------------
@@ -2541,7 +2859,7 @@ void P18F6x20::create_sfr_map()
 
 
   // cout << "Setting CCP cross-links\n";
-  ccp2con.setCrosslinks(&ccpr2l, &pir2, PIR2v2::CCP2IF, &tmr2);
+  ccp2con.setCrosslinks(&ccpr2l, pir2, PIR2v2::CCP2IF, &tmr2);
   ccp2con.setIOpin(&((*m_portc)[1]));
   ccpr2l.ccprh  = &ccpr2h;
   ccpr2l.tmrl   = &tmr1l;
