@@ -54,7 +54,7 @@ License along with this library; if not, see
 #include "p16x7x.h"
 #include "p12x.h"
 #include "p12f6xx.h"
-#include "p12f182x.h"
+#include "p1xf1xxx.h"
 #ifdef P17C7XX  // code no longer works
 #include "p17c75x.h"
 #endif
@@ -86,7 +86,6 @@ protected:
 ClockPhase *mCurrentPhase=0;
 phaseExecute1Cycle *mExecute1Cycle=0;
 phaseExecute2ndHalf *mExecute2ndHalf=0;     // misnomer - should be 2-cycle
-phaseExecuteInterrupt *mExecuteInterrupt=0;
 phaseCaptureInterrupt *mCaptureInterrupt=0;
 phaseIdle *mIdle=0;
 
@@ -97,19 +96,19 @@ void setCurrentPhase(ClockPhase *pPhase)
 
 const char* phaseDesc(ClockPhase *pPhase)
 {
+  if (pPhase == mIdle)
+    return ("mIdle");
   if (pPhase == mExecute1Cycle)
     return ("mExecute1Cycle");
   if (pPhase == mExecute2ndHalf)
     return ("mExecute2ndHalf");
-  if (pPhase == mExecuteInterrupt)
-    return ("mExecuteInterrupt");
   if (pPhase == mCaptureInterrupt)
     return ("mCaptureInterrupt");
   return "unknown phase";
 }
 
 phaseCaptureInterrupt::phaseCaptureInterrupt(Processor *pcpu)
-  :  ProcessorPhase(pcpu)
+  :  ProcessorPhase(pcpu), m_pCurrentPhase(0),m_pNextNextPhase(0)
 {
 }
 phaseCaptureInterrupt::~phaseCaptureInterrupt()
@@ -122,10 +121,34 @@ ClockPhase *phaseCaptureInterrupt::advance()
   if (m_pNextPhase == mExecute2ndHalf)
     m_pNextPhase->advance();
 
+  if (m_pCurrentPhase == mIdle) { // Interrupted sleep
+
+    // complete sleeping phase
+    m_pNextPhase = m_pNextNextPhase->advance();
+    
+    if (m_pNextPhase == mIdle)
+    {
+        m_pNextPhase = mExecute1Cycle;
+	do 
+	{
+	    m_pNextPhase = mExecute1Cycle->advance();
+	}while (m_pNextPhase != mExecute1Cycle);
+    }
+    mCurrentPhase = this;
+    if (bp.global_break)
+	m_pNextNextPhase = m_pNextPhase;
+    else
+      m_pCurrentPhase = NULL;
+    
+    ((pic_processor *)m_pcpu)->exit_sleep();
+    return this;
+  }
+
   m_pcpu->interrupt();
 
   return m_pNextPhase;
 }
+
 void phaseCaptureInterrupt::firstHalf()
 {
   m_pCurrentPhase = mCurrentPhase;
@@ -223,6 +246,8 @@ ProcessorConstructor pP16CR83(P16CR83::construct ,
                               "__16CR83",   "pic16cr83",    "p16cr83",  "16cr83");
 ProcessorConstructor pP16CR84(P16CR84::construct ,
                               "__16CR84",   "pic16cr84",    "p16cr84",  "16cr84");
+ProcessorConstructor pP16F505(P16F505::construct ,
+                              "__16F505",   "pic16f505",    "p16f505",  "16f505");
 ProcessorConstructor pP16F73(P16F73::construct ,
                              "__16F73",     "pic16f73",     "p16f73",   "16f73");
 ProcessorConstructor pP16F74(P16F74::construct ,
@@ -297,6 +322,8 @@ ProcessorConstructor pP16F876A(P16F876A::construct ,
                               "__16F876a",  "pic16f876a",   "p16f876a", "16f876a");
 ProcessorConstructor pP16F877A(P16F877A::construct ,
                               "__16F877a",  "pic16f877a",   "p16f877a", "16f877a");
+ProcessorConstructor pP16F1788(P16F1788::construct ,
+                              "__16F1788", "pic16f1788", "p16f1788", "16f1788");
 ProcessorConstructor pP16F1823(P16F1823::construct ,
                               "__16F1823", "pic16f1823", "p16f1823", "16f1823");
 #ifdef P17C7XX  // code no longer works
@@ -345,10 +372,12 @@ ProcessorConstructor pP18F2321(P18F2321::construct,
                               "__18F2321",  "pic18f2321",   "p18f2321", "18f2321");
 ProcessorConstructor pP18F2420(P18F2420::construct,
                               "__18F2420",  "pic18f2420",   "p18f2420", "18f2420");
-ProcessorConstructor pP18F2520(P18F2520::construct,
-                              "__18F2520",  "pic18f2520",   "p18f2520", "18f2520");
 ProcessorConstructor pP18F2455(P18F2455::construct,
                               "__18F2455",  "pic18f2455",   "p18f2455", "18f2455");
+ProcessorConstructor pP18F2520(P18F2520::construct,
+                              "__18F2520",  "pic18f2520",   "p18f2520", "18f2520");
+ProcessorConstructor pP18F2550(P18F2550::construct,
+                              "__18F2550",  "pic18f2550",   "p18f2550", "18f2550");
 ProcessorConstructor pP18F4221(P18F4221::construct,
                               "__18F4221",  "pic18f4221",   "p18f4221", "18f4221");
 ProcessorConstructor pP18F4321(P18F4321::construct,
@@ -357,6 +386,8 @@ ProcessorConstructor pP18F4420(P18F4420::construct,
                               "__18F4420",  "pic18f4420",   "p18f4420", "18f4420");
 ProcessorConstructor pP18F4520(P18F4520::construct,
                               "__18F4520",  "pic18f4520",   "p18f4520", "18f4520");
+ProcessorConstructor pP18F4550(P18F4550::construct,
+                              "__18F4550",  "pic18f4550",   "p18f4550", "18f4550");
 ProcessorConstructor pP18F4455(P18F4455::construct,
                               "__18F4455",  "pic18f4455",   "p18f4455", "18f4455");
 ProcessorConstructor pP18F4620(P18F4620::construct,
@@ -1069,9 +1100,9 @@ pic_processor::pic_processor(const char *_name, const char *_desc)
     m_MCLR(0), m_MCLR_Save(0), m_MCLRMonitor(0)
 {
 
+
    mExecute1Cycle    = new phaseExecute1Cycle(this);
    mExecute2ndHalf   = new phaseExecute2ndHalf(this);
-   mExecuteInterrupt = new phaseExecuteInterrupt(this);
    mCaptureInterrupt = new phaseCaptureInterrupt(this);
    mIdle             = new phaseIdle(this);
    mCurrentPhase   = mExecute1Cycle;
@@ -1097,6 +1128,14 @@ pic_processor::pic_processor(const char *_name, const char *_desc)
 //-------------------------------------------------------------------
 pic_processor::~pic_processor()
 {
+
+  if (pma)
+  {
+    while(!rma.SpecialRegisters.empty())
+      rma.SpecialRegisters.pop_back();
+    while(!pma->SpecialRegisters.empty())
+      pma->SpecialRegisters.pop_back();
+  }
   delete m_pResetTT;
   delete m_pInterruptTT;
 
@@ -1111,7 +1150,6 @@ pic_processor::~pic_processor()
 
   delete mExecute1Cycle;
   delete mExecute2ndHalf;
-  delete mExecuteInterrupt;
   delete mCaptureInterrupt;
   delete mIdle;
 
